# coding=utf-8

RPC_REPLY_QUEUE_TEMPLATE = 'rpc.reply-{}-{}'
RPC_REPLY_QUEUE_TTL = 300000  # ms (5 mins)
RPC_EXCHANGE_CONFIG_KEY = 'rpc_exchange'
RPC_EXCHANGE_NAME = 'nameko-rpc'

DEFAULT_TIMEOUT = 30
DEFAULT_CON_TIMEOUT = 3



# coding=utf-8
from __future__ import absolute_import
import re
import aiotask_context as ctx
from aio_nameko_proxy import AIOPooledClusterRpcProxy


class SanicNamekoClusterRpcProxy(AIOPooledClusterRpcProxy):

    def __init__(self, app=None):
        if app:
            self.init_app(app)

    def init_app(self, app):
        config = dict()
        for k, v in app.config.items():
            match = re.match(r"NAMEKO_(?P<name>.*)", k)
            if match:
                name = match.group("name")
                if name == "AMQP_URI":
                    config[name] = v
                    continue
                name = name.lower()
                config[name] = v
        self.parse_config(config)

        @app.listener('after_server_start')
        async def _set_aiotask_factory(app, loop):
            loop.set_task_factory(ctx.task_factory)

        @app.listener('after_server_start')
        async def _init_proxy_pool(app, loop):
            self.loop = loop
            await self.init_pool()

        @app.listener('before_server_stop')
        async def _close_proxy_pool(app, loop):
            await self.close()

        @app.middleware('response')
        async def release_nameko_proxy(request, response):
            proxy = ctx.get('_nameko_rpc_proxy', None)
            if proxy is not None:
                self.release_proxy(proxy)

    async def get_proxy(self):
        proxy = ctx.get('_nameko_rpc_proxy', None)
        if not proxy:
            proxy = await super(SanicNamekoClusterRpcProxy, self).get_proxy()
            ctx.set('_nameko_rpc_proxy', proxy)
        return proxy




# coding=utf-8

from .excs import *
from .proxies import AIOClusterRpcProxy, AIOPooledClusterRpcProxy

__author__ = "laiyongtao <laiyongtao6908@163.com>"

version_info = (1, 0, 0)
__version__ = ".".join(map(str, version_info))


import os
from setuptools import setup, find_packages

Dir = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(Dir, "README.md")) as f:
    long_desc = f.read()

desc = '''A standalone nameko rpc proxy for asyncio and a wrapper for using nameko rpc proxy with Sanic. '''


setup(
    name="aio-nameko-proxy",
    version="1.0.0",
    author="laiyongtao",
    author_email="laiyongtao6908@163.com",
    url="https://github.com/laiyongtao/aio_nameko_proxy" ,
    license="Apache License, Version 2.0",
    description=desc,
    long_description=long_desc,
    long_description_content_type="text/markdown",
    platforms="all",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    keywords = ("nameko", "sanic", "asyncio", "rpc"),
    packages=find_packages(exclude=["demos"]),
    install_requires=[
        "aio-pika>=6.6.0",
        "aiotask-context>=0.6.1",
    ],
    python_requires=">3.5.*, <4",
)



# coding=utf-8

class ConfigError(Exception):
    pass

class ClientError(Exception):
    pass

class RemoteError(Exception):
    """ Exception to raise at the caller if an exception occurred in the
    remote worker.
    """

    def __init__(self, exc_type=None, value=""):
        self.exc_type = exc_type
        self.value = value
        message = '{} {}'.format(exc_type, value)
        super(RemoteError, self).__init__(message)


def deserialize(data):
    """ Deserialize `data` to an exception instance.

    If the `exc_path` value matches an exception registered as
    ``deserializable``, return an instance of that exception type.
    Otherwise, return a `RemoteError` instance describing the exception
    that occurred.
    """

    exc_type = data.get('exc_type')
    value = data.get('value')
    return RemoteError(exc_type=exc_type, value=value)
